@@ -110,7 +110,7 @@ static int lower_correct (struct buffer_head * bh, struct item_head * lower,
 
 
 /* return 1 if something was changed */
-static int correct_key_format (struct item_head * ih)
+static int correct_key_format (struct item_head * ih, int symlink)
 {
     int dirty = 0;
 
@@ -139,6 +139,12 @@ static int correct_key_format (struct item_head * ih)
 	die ("stat_data item of the wrong length");
     }
     
+    if (symlink && is_direct_ih(ih) && (key_format(&ih->ih_key) != KEY_FORMAT_1)) {	
+	/* All symlinks are of 3.5 format */
+	/*fsck_log ("correct_key_format: Symlink keys should be of 3.5 format. %k - fixed.\n", &ih->ih_key); */
+	set_type_and_offset(KEY_FORMAT_1, &ih->ih_key, get_offset(&ih->ih_key), get_type(&ih->ih_key));
+    }
+	
     if (key_format (&ih->ih_key) != get_ih_key_format (ih)) {
 	/*fsck_log ("correct_key_format: ih_key_format of (%H) is set to format found in the key\n",
 	  ih);*/
@@ -532,14 +538,13 @@ static int verify_directory_item (reiserfs_filsys_t * fs, struct buffer_head * b
 }
 
 
-static int does_it_fit_into_dev (__u64 offset) {
+static __inline__ int does_it_fit_into_dev (__u64 offset, __u64 fs_size) {
 /* 
    Count of unformatted pointers - offset / blocksize
    Count of blocks to store them - UNFM_P_SIZE * offset / blocksize / MAX_ITEM_LEN
    Size to store it              - blocksize * UNFM_P_SIZE * offset / blocksize / MAX_ITEM_LEN
 */
-   return ( UNFM_P_SIZE * offset / MAX_ITEM_LEN(fs->fs_blocksize) < 
-	fs->fs_blocksize * get_sb_block_count (fs->fs_ondisk_sb)) ? 1 : 0;
+   return ( UNFM_P_SIZE * offset / MAX_ITEM_LEN(fs->fs_blocksize) < fs_size) ? 1 : 0;
 }
 
 
@@ -571,7 +576,7 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
     int file_format = KEY_FORMAT_UNDEFINED;
     int symlnk = 0;
     int bad_order;
-
+    __u64 fs_size;
 
     /* Delete all safe links. */
     for (i = get_blkh_nr_items (B_BLK_HEAD (bh)) - 1; i >= 0; i--) {
@@ -580,6 +585,8 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
 	}
     }
 
+    fs_size = (__u64)fs->fs_blocksize * get_sb_block_count (fs->fs_ondisk_sb);
+
  start_again:
 
     ih = B_N_PITEM_HEAD (bh, 0);
@@ -605,7 +612,6 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
             }
         }
 
-
 	if (is_wrong_short_key (&ih->ih_key)) {
 	    /* sometimes stat datas get k_objectid==0 or k_dir_id==0 */
 	    
@@ -807,15 +813,15 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
             int err = 0;
             if (i > 1) {
                 if (comp_short_keys (&(ih - 2)->ih_key, &(ih - 1)->ih_key) != -1)
-                    set_bit (1, &err);
+                    misc_set_bit (1, &err);
                 if (comp_short_keys (&(ih - 2)->ih_key, &ih->ih_key) != -1)
-                    set_bit (2, &err);
+                    misc_set_bit (2, &err);
             }
             if (i + 1 < nr_items) {
                 if (comp_short_keys (&(ih - 1)->ih_key, &(ih + 1)->ih_key) != -1)
-                    set_bit (3, &err);
+                    misc_set_bit (3, &err);
                 if (comp_short_keys (&ih->ih_key, &(ih + 1)->ih_key) != -1)
-                    set_bit (4, &err);
+                    misc_set_bit (4, &err);
             }
 /*
             if ((test_bit (1, err) || test_bit (3, err)) &&
@@ -826,8 +832,8 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
                 goto start_again;
       	    }
 */
-            if (!test_bit (1, &err) && !test_bit (3, &err) &&
-		!test_bit (2, &err) && !test_bit (4, &err)) {
+            if (!misc_test_bit (1, &err) && !misc_test_bit (3, &err) &&
+		!misc_test_bit (2, &err) && !misc_test_bit (4, &err)) {
       		if (i <= 1) {
             	    /* take bigger */
                     if (comp_short_keys (&(ih - 1)->ih_key, &ih->ih_key) == 1) {
@@ -856,14 +862,14 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
                     }
                 }
 		dirty = 1;
-	    } else if (!test_bit (1, &err) && !test_bit (3, &err)) {
+	    } else if (!misc_test_bit (1, &err) && !misc_test_bit (3, &err)) {
       		/* take i - 1 */
 		fsck_log("pass0: vpf-10590: block %lu, item %d: Wrong order of items - "
 		    "change the object_id of the key %k to %lu\n",
                     bh->b_blocknr, i, &ih->ih_key, get_key_objectid (&(ih - 1)->ih_key));
 		set_key_objectid (&ih->ih_key, get_key_objectid (&(ih - 1)->ih_key));
 		dirty = 1;
- 	    } else if (!test_bit (2, &err) && !test_bit (4, &err)) {
+ 	    } else if (!misc_test_bit (2, &err) && !misc_test_bit (4, &err)) {
       		/* take i */
 		fsck_log("pass0: vpf-10600: block %lu, item %d: Wrong order of items - "
 		    "change the object_id of the key %k to %lu\n",
@@ -993,9 +999,7 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
 			delete_item (fs, bh, i);
 			goto start_again;
 		    }
-		}
-		
-		if (is_direct_ih (ih)) {
+		} else if (is_direct_ih (ih)) {
 		    if (!correct_direct_item_offset (get_offset (&ih->ih_key), key_format (&ih->ih_key)) ||
           		STORE_TAIL_IN_UNFM (offset + get_ih_item_len (ih) - 1, get_ih_item_len (ih), bh->b_size)) {
                             fsck_log("pass0: vpf-10210: block %lu, item %d: The item with wrong offset ", 
@@ -1007,7 +1011,7 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
   		}
 		
 		offset += get_bytes_number (ih, fs->fs_blocksize);
-		if (!does_it_fit_into_dev (offset)) {
+		if (!does_it_fit_into_dev (offset, fs_size)) {
                     fsck_log("pass0: vpf-10230: block %lu, item %d: The item offset is is too big %k - deleted\n",
                                         bh->b_blocknr, i, &ih->ih_key);
                     delete_item (fs, bh, i);
@@ -1054,6 +1058,7 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
 
 		default:
 		    /* upper item was the first item of a node */
+		    /* to make gcc 3.2 do not sware here */;
 		}
 
 		retval = lower_correct (bh, ih, i);
@@ -1073,8 +1078,8 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
 		    goto start_again;
 
 		default:
-		    /* there were only two items in a node, so we could not
-                       decide what to delete, go and ask user */
+		    /* only 2 items in the node, how to decide what to delete, go and ask user */
+		    /* to make gcc 3.2 do not sware here */;
 		}
 		fsck_log ("pass0: block %lu, items %d and %d: Which of these items looks better (the other will be deleted.)?\n"
 			  "%k\n%k\n", bh->b_blocknr, i-1, i, &(ih - 1)->ih_key, &ih->ih_key);
@@ -1094,7 +1099,7 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
 	    goto start_again;
 	}
 
-	dirty += correct_key_format (ih);
+	dirty += correct_key_format (ih, symlnk);
 	
 	if (is_stat_data_ih (ih)) {
 	    __u16 mode;
@@ -1112,9 +1117,11 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
             fsck_log("pass0: vpf-10240: block %lu, item (%d): Item %k, which format (%d) is not equal to StatData "
 		"format (%d), is deleted\n", bh->b_blocknr, i, &ih->ih_key, get_ih_key_format(ih), file_format);
             delete_item (fs, bh, i);
-	    file_format = KEY_FORMAT_UNDEFINED;
             goto start_again;
-        }
+        } else {
+	    file_format = KEY_FORMAT_UNDEFINED;
+	    symlnk = 0;
+	}
 
 
 	if (i && is_stat_data_ih (ih - 1) && !not_of_one_file (&ih->ih_key, &(ih - 1)->ih_key)) {
@@ -1445,11 +1452,18 @@ static void do_pass_0 (reiserfs_filsys_t * fs)
 	/* just to test pass0_correct_leaf */
 	bh = bread (fs->fs_dev, fsck_data (fs)->rebuild.test, fs->fs_blocksize);
 
-	/*
+	if (!bh) {
+	    /* we were reading one block at time, and failed, so mark block bad */
+	    fsck_progress ("%s: Reading of the block %lu failed\n", __FUNCTION__, 
+		fsck_data (fs)->rebuild.test);
+	    reiserfs_free (fs);
+	    exit(0);
+	}
+
 	if (is_leaf_bad (bh)) {
 	    fsck_progress ("###############  bad #################\n");
 	}
-	*/
+
 	pass0_correct_leaf (fs, bh);
 	
 	print_block (stdout, fs, bh, 3, -1, -1);
@@ -1682,7 +1696,7 @@ static void init_source_bitmap (reiserfs_filsys_t * fs)
     case EXTERN_BITMAP:
 	fp = fopen (fsck_data (fs)->rebuild.bitmap_file_name, "r");
 	if (!fp)
-	    reiserfs_panic ("Could not load bitmap: %m\n");
+	    reiserfs_panic ("Could not load bitmap: %s\n", strerror(errno));
 	fsck_source_bitmap (fs) = reiserfs_bitmap_load (fp);
 	if (!fsck_source_bitmap (fs))
 	    reiserfs_panic ("Could not load fitmap from \"%s\"", 
@@ -1778,7 +1792,7 @@ static void before_pass_0 (reiserfs_filsys_t * fs)
     make_aux_bitmaps (fs);
 
     /* on pass0 all objectids will be marked here as used */
-    proper_id_map (fs) = init_id_map ();
+    proper_id_map (fs) = init_id_map (MAP_PACKED);
 
     /* pass0 gathers statistics about hash hits */
     hash_hits_init (fs);
