@@ -167,8 +167,8 @@ struct reiserfs_super_block_v1
                                    
 
 /* values for sb_mount_state field */
-#define REISERFS_CLEANLY_UMOUNTED    1 /* this was REISERFS_VALID_FS */
-#define REISERFS_NOT_CLEANLY_UMOUNTED    2 /* this was REISERFS_ERROR. It
+#define FS_CLEANLY_UMOUNTED    1 /* this was REISERFS_VALID_FS */
+#define FS_NOT_CLEANLY_UMOUNTED    2 /* this was REISERFS_ERROR. It
                                               means that filesystem was not
                                               cleanly unmounted */
 
@@ -244,9 +244,10 @@ typedef enum {
 #define get_sb_v2_inode_generation(sb)		get_le32 (sb, sb_inode_generation)
 #define set_sb_v2_inode_generation(sb,val)	set_le32 (sb, sb_inode_generation, val)
 
-//#define get_sb_v2_flag(sb, flag)	test_bit ((flag), &((struct reiserfs_super_block *)sb)->s_flags)	
-//#define set_sb_v2_flag(sb, flag)	set_bit  ((flag), &((struct reiserfs_super_block *)sb)->s_flags)
-#define get_sb_v2_flag(sb, flag)	(get_le32 (sb, s_flags) & flag)
+#define get_sb_v2_flags(sb)		get_le32 (sb, s_flags)
+#define set_sb_v2_flags(sb, val)	set_le32 (sb, s_flags, val)
+
+#define get_sb_v2_flag(sb, flag)       (get_le32 (sb, s_flags) & flag)
 #define set_sb_v2_flag(sb, flag)	set_le32 (sb, s_flags, get_le32 (sb, s_flags) | flag)
 #define clear_sb_v2_flag(sb, flag)	set_le32 (sb, s_flags, get_le32 (sb, s_flags) & ~(flag))
 
@@ -255,22 +256,20 @@ typedef enum {
 */
 
 /* these are possible values for sb_fs_state */
-#define REISERFS_CONSISTENT   0   /* this is set by mkreiserfs and by reiserfsck */
-#define REISERFS_CORRUPTED  0x1   /* this will be set by kernel code when it
-                                   encounters suspicious metadata */
-#define PASS_0_DONE      0xfaa1   /* set by fsck when pass-by-pass (-d),
-                                     REISERFS_CORRUPTED flag included */
-#define PASS_1_DONE      0xfaa3   /* set by fsck when pass-by-pass (-d),
-                                     REISERFS_CORRUPTED flag included */
-#define TREE_IS_BUILT    0xfaa5   /* set by fsck when pass-by-pass (-d),
-                                     REISERFS_CORRUPTED flag included */
-#define SEMANTIC_DONE    0xfaa7   /* set by fsck when pass-by-pass (-d),
-                                     REISERFS_CORRUPTED flag included */
-#define LOST_FOUND_DONE  0xfaa9   /* set by fsck when pass-by-pass (-d),
-                                     REISERFS_CORRUPTED flag included */
-//#define FIX_FIXABLE_IS_ALLOWED 0xfaa7
-
-
+#define FS_CONSISTENT   0x0	    /* this is set by mkreiserfs and by reiserfsck */
+#define FS_ERROR	0x1	    /* this is set by the kernel when fsck is wanted. */
+#define FS_FATAL	0x2	    /* this is set by fsck when fatal corruption is found */
+#define IO_ERROR	0x4	    /* this is set by kernel when io error occures */
+#define PASS_0_DONE     0xfa02	    /* set by fsck when pass-by-pass (-d),
+				       FS_FATAL flag included */
+#define PASS_1_DONE     0xfb02	    /* set by fsck when pass-by-pass (-d),
+                                       FS_FATAL flag included */
+#define TREE_IS_BUILT   0xfc02	    /* set by fsck when pass-by-pass (-d),
+                                       FS_FATAL flag included */
+#define SEMANTIC_DONE   0xfd02	    /* set by fsck when pass-by-pass (-d),
+                                       FS_FATAL flag included */
+#define LOST_FOUND_DONE 0xfe02	    /* set by fsck when pass-by-pass (-d),
+                                       FS_FATAL flag included */
 
 /* struct stat_data* access macros */
 /* v1 */
@@ -426,6 +425,8 @@ struct reiserfs_journal_header {
     __u32 jh_mount_id ;
 
     struct journal_params jh_journal;
+
+    __u32 jh_last_check_mount_id;	/* the mount id of the fs during the last reiserfsck --check. */
 };
 
 
@@ -492,11 +493,9 @@ struct offset_v2 {
 } __attribute__ ((__packed__));
 
 
-/* Key of the object drop determines its location in the S+tree, and is
-   composed of 4 components */
+/* Key of the object determines object's location in the tree, composed of 4 components */
 struct key {
-    __u32 k2_dir_id;    /* packing locality: by default parent directory object
-                          id */
+    __u32 k2_dir_id;    /* packing locality: by default parent directory object id */
     __u32 k2_objectid;  /* object identifier */
     union {
 	struct offset_v1 k2_offset_v1;
@@ -555,6 +554,8 @@ struct key {
 
 #define TYPE_UNKNOWN 15
 
+/* special type for symlink not conflicting to any of item types. */
+#define TYPE_SYMLINK	4
 
 #define KEY_FORMAT_1 0
 #define KEY_FORMAT_2 1
@@ -640,9 +641,11 @@ void set_ih_key_format (struct item_head * ih, __u16 val);
     ( ! not_of_one_file(p_s_ih, p_s_key) && \
           I_OFF_BYTE_IN_ITEM(p_s_ih, get_offset (p_s_key), n_blocksize) )
 
-#define IH_Bad 0
-#define IH_Unreachable 1
-#define IH_WAS_TAIL 2
+#define IH_Unreachable 0
+#define IH_Was_Tail    1
+#define IH_Checked     2
+#define IH_Writable    3
+
 
 /* Bad item flag is set temporary by recover_leaf */
 /*
@@ -670,16 +673,23 @@ extern_inline __u16 unmark_item_bad( struct item_head *ih )
 /* Unreachable bit is set on tree rebuilding and is cleared in semantic pass */
 #define clean_ih_flags(ih) set_ih_flags (ih, 0)
 
-#define ih_reachable(ih) (!(get_ih_flags (ih) & (1 << IH_Unreachable)))
+#define ih_reachable(ih)        (!(get_ih_flags (ih) & (1 << IH_Unreachable)))
 #define mark_ih_reachable(ih)   set_ih_flags (ih, get_ih_flags (ih) & ~(1 << IH_Unreachable))
 #define mark_ih_unreachable(ih) set_ih_flags (ih, get_ih_flags (ih) | (1 << IH_Unreachable))
 
-#define ih_was_tail(ih) (get_ih_flags (ih) & (1 << IH_WAS_TAIL))
-#define mark_ih_was_tail(ih) set_ih_flags (ih, get_ih_flags (ih) | (1 << IH_WAS_TAIL))
-#define mark_ih_become_tail(ih) set_ih_flags (ih, get_ih_flags (ih) & ~(1 << IH_WAS_TAIL))
+#define ih_was_tail(ih)         (get_ih_flags (ih) & (1 << IH_Was_Tail))
+#define mark_ih_was_tail(ih)    set_ih_flags (ih, get_ih_flags (ih) | (1 << IH_Was_Tail))
+#define mark_ih_become_tail(ih) set_ih_flags (ih, get_ih_flags (ih) & ~(1 << IH_Was_Tail))
 
+#define ih_checked(ih)          (get_ih_flags (ih) & (1 << IH_Checked))
+#define mark_ih_checked(ih)     set_ih_flags (ih, get_ih_flags (ih) | (1 << IH_Checked))
+#define clear_ih_checked(ih)    set_ih_flags (ih, get_ih_flags (ih) & ~(1 << IH_Checked))
 
-/* maximal length of item */ 
+#define ih_writable(ih)         (get_ih_flags (ih) & (1 << IH_Writable))
+#define mark_ih_writable(ih)    set_ih_flags (ih, get_ih_flags (ih) | (1 << IH_Writable))
+#define clear_ih_writable(ih)   set_ih_flags (ih, get_ih_flags (ih) & ~(1 << IH_Writable))
+
+/* maximal length of item */
 #define MAX_ITEM_LEN(block_size) (block_size - BLKH_SIZE - IH_SIZE)
 #define MIN_ITEM_LEN 1
 
@@ -690,7 +700,7 @@ extern_inline __u16 unmark_item_bad( struct item_head *ih )
 
 
 /* 
- * Picture represents a leaf of the S+tree
+ * Picture represents a leaf of internal tree
  *  ______________________________________________________
  * |      |  Array of     |                   |           |
  * |Block |  Object-Item  |      F r e e      |  Objects- |
@@ -837,7 +847,7 @@ struct stat_data {
 //
 #define SD_SIZE (sizeof(struct stat_data))
 
-// there are two ways: to check length of item or ih_version field
+// there are two ways: to check length of item or ih_format field
 // (for old stat data it is set to 0 (KEY_FORMAT_1))
 #define stat_data_v1(ih) (get_ih_key_format (ih) == KEY_FORMAT_1)
 
@@ -1053,12 +1063,12 @@ struct path_element  {
 };
 
 
-#define MAX_HEIGHT 5 /* maximal height of a tree. don't change this without changing JOURNAL_PER_BALANCE_CNT */
-#define EXTENDED_MAX_HEIGHT         7 /* Must be equals MAX_HEIGHT + FIRST_PATH_ELEMENT_OFFSET */
-#define FIRST_PATH_ELEMENT_OFFSET   2 /* Must be equal to at least 2. */
+#define MAX_HEIGHT 6
+#define FIRST_PATH_ELEMENT_OFFSET   2
+#define EXTENDED_MAX_HEIGHT         (MAX_HEIGHT + FIRST_PATH_ELEMENT_OFFSET)
 
-#define ILLEGAL_PATH_ELEMENT_OFFSET 1 /* Must be equal to FIRST_PATH_ELEMENT_OFFSET - 1 */
-#define MAX_FEB_SIZE 6   /* this MUST be MAX_HEIGHT + 1. See about FEB below */
+#define ILLEGAL_PATH_ELEMENT_OFFSET 1
+#define MAX_FEB_SIZE (MAX_HEIGHT + 1)
 
 
 
@@ -1127,21 +1137,21 @@ struct path var = {ILLEGAL_PATH_ELEMENT_OFFSET, }
 
 
 // search_by_key (and clones) and fix_nodes error code
-#define CARRY_ON          	0
-#define IO_ERROR		3
+#define CARRY_ON		    0
 
-#define NO_DISK_SPACE           4
-#define NO_BALANCING_NEEDED     5
-#define ITEM_FOUND              6
-#define ITEM_NOT_FOUND          7
-#define GOTO_PREVIOUS_ITEM      10
-#define POSITION_FOUND_INVISIBLE 11
-#define FILE_NOT_FOUND          12
+#define NO_DISK_SPACE		    3
+/* #define IO_ERROR		    0x4 - defined above as 0x4 */
+#define NO_BALANCING_NEEDED	    5
+#define ITEM_FOUND		    6
+#define ITEM_NOT_FOUND		    7
+#define GOTO_PREVIOUS_ITEM	    10
+#define POSITION_FOUND_INVISIBLE    11
+#define FILE_NOT_FOUND		    12
 
 // used by fsck
-#define DIRECTORY_NOT_FOUND     13 
-#define REGULAR_FILE_FOUND     14
-#define DIRECTORY_FOUND        15
+#define DIRECTORY_NOT_FOUND	    13 
+#define REGULAR_FILE_FOUND	    14
+#define DIRECTORY_FOUND		    15
 
 
 struct unfm_nodeinfo {
@@ -1424,7 +1434,8 @@ struct buffer_info {
 /* get stat-data */
 #define B_I_STAT_DATA(bh, ih) ( (struct stat_data * )B_I_PITEM(bh,ih) )
 
-#define MAX_DIRECT_ITEM_LEN(size) ((size) - BLKH_SIZE - 2*IH_SIZE - SD_V1_SIZE - UNFM_P_SIZE)
+#define MAX_DIRECT_ITEM_LEN(size) ((size) - BLKH_SIZE - 2*IH_SIZE - SD_SIZE - UNFM_P_SIZE)
+#define MAX_INDIRECT_ITEM_LEN(size) ((size) - BLKH_SIZE - IH_SIZE)
 
 /* indirect items consist of entries which contain blocknrs, pos
    indicates which entry, and B_I_POS_UNFM_POINTER resolves to the
@@ -1522,7 +1533,7 @@ void make_empty_leaf (struct buffer_head *);
 struct buffer_head * get_FEB (struct tree_balance *);
 
 
-__u64 get_bytes_number (struct item_head * ih, int blocksize);
+__u32 get_bytes_number (struct item_head * ih, int blocksize);
 
 
 
