@@ -127,8 +127,8 @@ void reiserfs_bitmap_disjunction (reiserfs_bitmap_t * to, reiserfs_bitmap_t * fr
 	    to->bm_bit_size == from->bm_bit_size);
 
     for (i = 0; i < to->bm_bit_size; i++) {
-	if (test_bit(i, from->bm_map) && !test_bit(i, to->bm_map)) {
-	    set_bit(i, to->bm_map);
+	if (misc_test_bit(i, from->bm_map) && !misc_test_bit(i, to->bm_map)) {
+	    misc_set_bit(i, to->bm_map);
 	    to->bm_set_bits ++;
 	    to->bm_dirty = 1;	
 	}
@@ -148,8 +148,8 @@ void reiserfs_bitmap_delta (reiserfs_bitmap_t * base, reiserfs_bitmap_t * exclud
 	    base->bm_bit_size == exclude->bm_bit_size);
 
     for (i = 0; i < base->bm_bit_size; i++) {
-	if (test_bit(i, exclude->bm_map) && test_bit(i, base->bm_map)) {
-	    clear_bit(i, base->bm_map);
+	if (misc_test_bit(i, exclude->bm_map) && misc_test_bit(i, base->bm_map)) {
+	    misc_clear_bit(i, base->bm_map);
 	    base->bm_set_bits --;
 	    base->bm_dirty = 1;
 	}
@@ -159,9 +159,9 @@ void reiserfs_bitmap_delta (reiserfs_bitmap_t * base, reiserfs_bitmap_t * exclud
 void reiserfs_bitmap_set_bit (reiserfs_bitmap_t * bm, unsigned int bit_number)
 {
     assert(bit_number < bm->bm_bit_size);
-    if (test_bit (bit_number, bm->bm_map))
+    if (misc_test_bit (bit_number, bm->bm_map))
 	return;
-    set_bit(bit_number, bm->bm_map);
+    misc_set_bit(bit_number, bm->bm_map);
     bm->bm_set_bits ++;
     bm->bm_dirty = 1;
 }
@@ -170,9 +170,9 @@ void reiserfs_bitmap_set_bit (reiserfs_bitmap_t * bm, unsigned int bit_number)
 void reiserfs_bitmap_clear_bit (reiserfs_bitmap_t * bm, unsigned int bit_number)
 {
     assert(bit_number < bm->bm_bit_size);
-    if (!test_bit (bit_number, bm->bm_map))
+    if (!misc_test_bit (bit_number, bm->bm_map))
 	return;
-    clear_bit (bit_number, bm->bm_map);
+    misc_clear_bit (bit_number, bm->bm_map);
     bm->bm_set_bits --;
     bm->bm_dirty = 1;
 }
@@ -183,7 +183,7 @@ int reiserfs_bitmap_test_bit (reiserfs_bitmap_t * bm, unsigned int bit_number)
     if (bit_number >= bm->bm_bit_size)
 	printf ("bit %u, bitsize %lu\n", bit_number, bm->bm_bit_size);
     assert(bit_number < bm->bm_bit_size);
-    return test_bit(bit_number, bm->bm_map);
+    return misc_test_bit(bit_number, bm->bm_map);
 }
 
 
@@ -204,7 +204,7 @@ int reiserfs_bitmap_find_zero_bit (reiserfs_bitmap_t * bm, unsigned long * start
     unsigned int  bit_nr = *start;
     assert(*start < bm->bm_bit_size);
 
-    bit_nr = find_next_zero_bit(bm->bm_map, bm->bm_bit_size, *start);
+    bit_nr = misc_find_next_zero_bit(bm->bm_map, bm->bm_bit_size, *start);
 
     if (bit_nr >= bm->bm_bit_size) { /* search failed */	
 	return 1;
@@ -220,13 +220,12 @@ static int reiserfs_fetch_ondisk_bitmap (reiserfs_bitmap_t * bm, reiserfs_filsys
 {
     unsigned long to_copy;
     int copied;
-    int i;
+    int i, ret = 0;
     char * p;
     int last_byte_unused_bits;
     unsigned long block;
     struct buffer_head * bh;
 
-
     to_copy = (get_sb_block_count (fs->fs_ondisk_sb) + 7) / 8;
 
     /*reiserfs_warning (stderr, "Fetching on-disk bitmap..");*/
@@ -248,8 +247,16 @@ static int reiserfs_fetch_ondisk_bitmap (reiserfs_bitmap_t * bm, reiserfs_filsys
 	    mark_buffer_uptodate (bh, 1);
 	}
 
-	if (to_copy < fs->fs_blocksize)
+	if (to_copy < fs->fs_blocksize) {
+	    for (i = to_copy; i < fs->fs_blocksize; i++) {
+		if (bh->b_data[i] != (char)0xff) {
+		    ret = 1;
+		    break;
+		}
+	    }
+	    
 	    copied = to_copy;
+	}
 	memcpy (p, bh->b_data, copied); 
 	brelse (bh);
 	p += copied;
@@ -265,8 +272,12 @@ static int reiserfs_fetch_ondisk_bitmap (reiserfs_bitmap_t * bm, reiserfs_filsys
     /* on disk bitmap has bits out of SB_BLOCK_COUNT set to 1, where as
        reiserfs_bitmap_t has those bits set to 0 */
     last_byte_unused_bits = bm->bm_byte_size * 8 - bm->bm_bit_size;
-    for (i = 0; i < last_byte_unused_bits; i ++)
-	clear_bit (bm->bm_bit_size + i, bm->bm_map);
+    for (i = 0; i < last_byte_unused_bits; i ++) {
+	if (misc_test_bit (bm->bm_bit_size + i, bm->bm_map) == 0)
+	    ret = 1;
+	else	    
+	    misc_clear_bit (bm->bm_bit_size + i, bm->bm_map);
+    }
 
     bm->bm_set_bits = 0;
     /* FIXME: optimize that */
@@ -275,7 +286,8 @@ static int reiserfs_fetch_ondisk_bitmap (reiserfs_bitmap_t * bm, reiserfs_filsys
 	    bm->bm_set_bits ++;
     
     bm->bm_dirty = 0;
-    return 0;
+
+    return ret;
 }
 
 
@@ -332,7 +344,7 @@ int reiserfs_flush_to_ondisk_bitmap (reiserfs_bitmap_t * bm, reiserfs_filsys_t *
 	    /* set unused bits of last byte of a bitmap to 1 */
 	    last_byte_unused_bits = bm->bm_byte_size * 8 - bm->bm_bit_size;
 	    for (i = 0; i < last_byte_unused_bits; i ++)
-		set_bit ((bm->bm_bit_size % (fs->fs_blocksize * 8)) + i, bh->b_data);
+		misc_set_bit ((bm->bm_bit_size % (fs->fs_blocksize * 8)) + i, bh->b_data);
 	}
 	mark_buffer_dirty (bh);
 	brelse (bh);
@@ -409,8 +421,8 @@ void reiserfs_bitmap_save (FILE * fp, reiserfs_bitmap_t * bm)
 
   /*  fp = fopen (filename, "w+");
     if (!fp) {
-	reiserfs_warning (stderr, "reiserfs_bitmap_save: could not save bitmap in %s: %m",
-			  filename);
+	reiserfs_warning (stderr, "reiserfs_bitmap_save: could not save bitmap in %s: %s",
+			  filename, strerror(errno));
 	return;
     }*/
 
@@ -430,7 +442,7 @@ void reiserfs_bitmap_save (FILE * fp, reiserfs_bitmap_t * bm)
     position = ftell(fp);
 
     if (fseek (fp, 4, SEEK_CUR)) {
-	reiserfs_warning (stderr, "reiserfs_bitmap_save: fseek failed: %m\n");
+	reiserfs_warning (stderr, "reiserfs_bitmap_save: fseek failed: %s\n", strerror(errno));
 //	fclose (fp);
 	return;
     }
@@ -481,14 +493,14 @@ void reiserfs_bitmap_save (FILE * fp, reiserfs_bitmap_t * bm)
     fwrite (&v, 4, 1, fp);
 
     if (fseek (fp, position, SEEK_SET)) {
-	reiserfs_warning (stderr, "reiserfs_bitmap_save: fseek failed: %m");
+	reiserfs_warning (stderr, "reiserfs_bitmap_save: fseek failed: %s", strerror(errno));
 	return;
     }
 
     fwrite (&extents, 4, 1, fp);
 
     if (fseek (fp, 0, SEEK_END)) {
-	reiserfs_warning (stderr, "reiserfs_bitmap_save: fseek failed: %m");
+	reiserfs_warning (stderr, "reiserfs_bitmap_save: fseek failed: %s", strerror(errno));
 	return;
     }
 }
@@ -528,7 +540,8 @@ int is_stage_magic_correct (FILE * fp)
     __u32 v;
 
     if (fseek (fp, -4, SEEK_END)) {
-	reiserfs_warning (stderr, "is_stage_magic_correct: fseek failed: %m\n");
+	reiserfs_warning (stderr, "is_stage_magic_correct: fseek failed: %s\n", 
+	    strerror(errno));
 	return -1;
     }
 
@@ -539,7 +552,8 @@ int is_stage_magic_correct (FILE * fp)
     }
 
     if (fseek (fp, 0, SEEK_SET)) {
-	reiserfs_warning (stderr, "is_stage_magic_correct: fseek failed: %m\n");
+	reiserfs_warning (stderr, "is_stage_magic_correct: fseek failed: %s\n", 
+	    strerror(errno));
 	return -1;
     }
 
@@ -571,7 +585,7 @@ reiserfs_bitmap_t * reiserfs_bitmap_load (FILE * fp)
     
 /*    fp = fopen (filename, "r");
     if (!fp) {
-	reiserfs_warning (stderr, "reiserfs_bitmap_load: fopen failed: %m\n");
+	reiserfs_warning (stderr, "reiserfs_bitmap_load: fopen failed: %s\n", strerror(errno));
 	return 0;
     }*/
 
@@ -659,33 +673,22 @@ void reiserfs_free_ondisk_bitmap (reiserfs_filsys_t * fs)
 /* read bitmap blocks */
 int reiserfs_open_ondisk_bitmap (reiserfs_filsys_t * fs)
 {
-    struct buffer_head * bh;
-
     if (fs->fs_bitmap2)
 	reiserfs_panic ("%s: bitmap is initiaized already", __FUNCTION__);
     fs->fs_bitmap2 = reiserfs_create_bitmap (get_sb_block_count (fs->fs_ondisk_sb));
     if (!fs->fs_bitmap2)
-	return 0;
+	return -1;
 
-    if ( (get_sb_block_count (fs->fs_ondisk_sb) + fs->fs_blocksize * 8 - 1) / (fs->fs_blocksize * 8) !=
-    	get_sb_bmap_nr (fs->fs_ondisk_sb)) {
+    if ((get_sb_block_count (fs->fs_ondisk_sb) + fs->fs_blocksize * 8 - 1) / 
+	(fs->fs_blocksize * 8) != get_sb_bmap_nr (fs->fs_ondisk_sb)) 
+    {
 	reiserfs_warning (stderr, "%s: wrong either bitmaps number,\n", __FUNCTION__);
-	reiserfs_warning (stderr, "count of blocks or blocksize, run with --rebuild-sb to fix it\n");
-	return 0;
+	reiserfs_warning (stderr, "count of blocks or blocksize, run with --rebuild-sb "
+	    "to fix it\n");
+	return -1;
     }    	
 
-    bh = bread (fs->fs_dev, fs->fs_bitmap2->bm_bit_size - 1, fs->fs_blocksize);
-    if (!bh) {
-	reiserfs_warning (stderr, "%s: bread failed for block %lu\n", __FUNCTION__,  fs->fs_bitmap2->bm_bit_size - 1);
-	reiserfs_warning (stderr, "\tYour partition is not big enough. Enlarge your partition or\n");
-	reiserfs_warning (stderr, "\trun reiserfsck with --rebuild-sb to fix super block.\n");
-	return 0;
-    }	
-
-    brelse (bh);
-    	
-    reiserfs_fetch_ondisk_bitmap (fs->fs_bitmap2, fs);
-    return 1;
+    return reiserfs_fetch_ondisk_bitmap (fs->fs_bitmap2, fs);
 }
 
 int reiserfs_create_ondisk_bitmap (reiserfs_filsys_t * fs)
