@@ -53,12 +53,57 @@ unsigned long next_desc_expected (reiserfs_filsys_t * fs, struct buffer_head * d
 	((offset + get_desc_trans_len (d_bh) + 2) % get_jp_journal_size (sb_jp));
 }
 
+/* common checks for validness of a transaction */
+int transaction_check_content (reiserfs_filsys_t * fs, reiserfs_trans_t * trans) {
+    struct buffer_head *d_bh, *c_bh;
+    struct reiserfs_journal_desc * desc;
+    struct reiserfs_journal_commit * commit;
+    unsigned long block;
+    int trans_half, i;
 
-/**/
-int is_valid_transaction (reiserfs_filsys_t * fs, struct buffer_head * d_bh)
-{
-    struct buffer_head * c_bh;
+    d_bh = bread (fs->fs_journal_dev, trans->desc_blocknr, fs->fs_blocksize);
+
+    if (!d_bh || who_is_this (d_bh->b_data, d_bh->b_size) != THE_JDESC)
+	goto error_desc_brelse;
+
+    /* read expected commit block and compare with descriptor block */
+    c_bh = bread (fs->fs_journal_dev, commit_expected (fs, d_bh), fs->fs_blocksize);
+    if (!c_bh)
+	goto error_desc_brelse;
+ 
+    if (!does_desc_match_commit (d_bh, c_bh)) 
+	goto error_commit_brelse;
+
+    /* Check that all target blocks are journalable */
+    desc = (struct reiserfs_journal_desc *)(d_bh->b_data);
+    commit = (struct reiserfs_journal_commit *)(c_bh->b_data);
+
+    trans_half = journal_trans_half (d_bh->b_size);
+    for (i = 0; i < get_desc_trans_len(d_bh); i++) {
+	if (i < trans_half)
+	    block = le32_to_cpu (desc->j2_realblock[i]);
+	else
+	    block = le32_to_cpu (commit->j3_realblock[i - trans_half]);
+
+	if (not_journalable(fs, block)) 
+	    goto error_commit_brelse;
+    }
     
+    brelse (d_bh);
+    brelse (c_bh);
+    return 1;
+    
+error_commit_brelse:
+    brelse (c_bh);
+error_desc_brelse:
+    brelse(d_bh);
+    return 0;
+}
+
+/* common checks for validness of a transaction */
+int transaction_check_desc(reiserfs_filsys_t * fs, struct buffer_head * d_bh) {
+    struct buffer_head * c_bh;
+    int ret = 1;
 
     if (!d_bh || who_is_this (d_bh->b_data, d_bh->b_size) != THE_JDESC)
 	return 0;
@@ -67,13 +112,12 @@ int is_valid_transaction (reiserfs_filsys_t * fs, struct buffer_head * d_bh)
     c_bh = bread (fs->fs_journal_dev, commit_expected (fs, d_bh), fs->fs_blocksize);
     if (!c_bh)
 	return 0;
-    
-    if (does_desc_match_commit (d_bh, c_bh)) {
-	brelse (c_bh);
-	return 1;
-    }
+ 
+    if (!does_desc_match_commit (d_bh, c_bh)) 
+	ret = 0;
+
     brelse (c_bh);
-    return 0;
+    return ret;
 }
 
 
@@ -102,7 +146,7 @@ int get_boundary_transactions (reiserfs_filsys_t * fs,
     trans_nr = 0;
     for (j_cur = 0; j_cur < j_size; j_cur ++) {
 	d_bh = bread (fs->fs_journal_dev, j_start + j_cur, fs->fs_blocksize);
-	if (!is_valid_transaction (fs, d_bh)) {
+	if (!transaction_check_desc (fs, d_bh)) {
 	    brelse (d_bh);
 	    continue;
 	}
@@ -160,7 +204,7 @@ int next_transaction (reiserfs_filsys_t * fs, reiserfs_trans_t * trans, reiserfs
 
     /* make sure that 'trans' is a valid transaction */
     d_bh = bread (fs->fs_journal_dev, trans->desc_blocknr, fs->fs_blocksize);
-    if (!is_valid_transaction (fs, d_bh))
+    if (!transaction_check_desc (fs, d_bh))
 	die ("next_transaction: valid transaction is expected");
 
     block = next_desc_expected (fs, d_bh);
@@ -168,7 +212,7 @@ int next_transaction (reiserfs_filsys_t * fs, reiserfs_trans_t * trans, reiserfs
 
     while (1) {
 	next_d_bh = bread (fs->fs_journal_dev, block, fs->fs_blocksize);
-	if (is_valid_transaction (fs, next_d_bh))
+	if (transaction_check_desc (fs, next_d_bh))
 	    break;
 
 	brelse (next_d_bh);
@@ -238,7 +282,7 @@ void for_each_block (reiserfs_filsys_t * fs, reiserfs_trans_t * trans,
     unsigned long block;
     int trans_half;
 
-    
+ 
     d_bh = bread (fs->fs_journal_dev, trans->desc_blocknr, fs->fs_blocksize);
     if (!d_bh) {
 	reiserfs_warning (stdout, "reading descriptor block %lu failed\n", trans->desc_blocknr);
@@ -295,7 +339,10 @@ void for_each_transaction (reiserfs_filsys_t * fs, action_on_trans_t action)
 
     while (1) {
 	action (fs, &oldest);	
-	if ((ret == TRANS_LAST) || ((ret = next_transaction (fs, &oldest, newest)) == TRANS_NOT_FOUND) )
+	if (ret == TRANS_LAST)
+	    break;
+	ret = next_transaction (fs, &oldest, newest);	
+	if (ret == TRANS_NOT_FOUND)
 	    break;
     }
 }
@@ -434,17 +481,19 @@ int reiserfs_open_journal (reiserfs_filsys_t * fs, char * j_filename, int flags)
     }
 	
     j_head = (struct reiserfs_journal_header *)(bh->b_data);
-	
-    if (is_reiserfs_jr_magic_string (sb)) {
-        if (get_jp_journal_1st_block (&j_head->jh_journal) != get_jp_journal_1st_block (sb_jp (sb)) || 
-            get_jp_journal_dev (&j_head->jh_journal) != get_jp_journal_dev (sb_jp (sb)) || 
-            get_jp_journal_size (&j_head->jh_journal) != get_jp_journal_size (sb_jp (sb))) 
-	{
-	    reiserfs_warning (stderr, "reiserfs_open_journal: no journal found on %s\n", j_filename);
-            brelse (bh);
-            return 0;
-        }
-    } else {
+
+    if (get_jp_journal_1st_block (&j_head->jh_journal) != get_jp_journal_1st_block (sb_jp (sb)) || 
+        get_jp_journal_dev (&j_head->jh_journal) != get_jp_journal_dev (sb_jp (sb)) || 
+        get_jp_journal_size (&j_head->jh_journal) != get_jp_journal_size (sb_jp (sb))) 
+    {
+	reiserfs_warning (stderr, "reiserfs_open_journal: journal parameters from super block does not match to\n"
+	    "journal parameters from journal.\n", j_filename);
+        brelse (bh);
+        return 0;
+        
+    }
+
+    if (!is_reiserfs_jr_magic_string (sb)) {    
 	if (get_jp_journal_dev (sb_jp(sb)) != 0 || 
 	    get_jp_journal_1st_block (sb_jp(sb)) != get_journal_start_must (fs) || 
 	    get_jp_journal_size (sb_jp(sb)) != journal_default_size(fs->fs_super_bh->b_blocknr, fs->fs_blocksize))
@@ -529,7 +578,7 @@ int reiserfs_create_journal (reiserfs_filsys_t * fs, char * j_filename,
 	}
 	if (stat (j_filename, &st) == -1) {
 		reiserfs_warning (stderr, "reiserfs_create_journal: "
-					  "stat %s failed: %m\n", j_filename);
+					  "stat %s failed: %s\n", j_filename, strerror(errno));
 		return 0;
 	}
 	if (!S_ISBLK (st.st_mode)) {
@@ -541,8 +590,8 @@ int reiserfs_create_journal (reiserfs_filsys_t * fs, char * j_filename,
 
     fs->fs_journal_dev = open (j_filename, O_RDWR | O_LARGEFILE);
     if (fs->fs_journal_dev == -1) {
-	reiserfs_warning (stderr, "reiserfs_create_journal: could not open %s: %m\n",
-			  j_filename);
+	reiserfs_warning (stderr, "reiserfs_create_journal: could not open %s: %s\n",
+			  j_filename, strerror(errno));
 	return 0;
     }
 
@@ -601,7 +650,7 @@ void reiserfs_reopen_journal (reiserfs_filsys_t * fs, int flag)
     flush_buffers (fs->fs_journal_dev);
     invalidate_buffers (fs->fs_journal_dev);
     if (close (fs->fs_journal_dev))
-	die ("reiserfs_reopen_journal: closed failed: %m");
+	die ("reiserfs_reopen_journal: closed failed: %s", strerror(errno));
 
     fs->fs_journal_dev = open (fs->fs_j_file_name, flag | O_LARGEFILE);
     if (fs->fs_journal_dev == -1)
@@ -665,7 +714,7 @@ int replay_journal (reiserfs_filsys_t * fs)
     struct buffer_head * bh;
     struct reiserfs_journal_header * j_head;
     reiserfs_trans_t cur, newest, control;
-    int replayed, ret = 0;
+    int replayed, ret = TRANS_NOT_FOUND;
 
     if (!reiserfs_journal_opened (fs))
         reiserfs_panic ("replay_journal: journal is not opened");
@@ -686,29 +735,42 @@ int replay_journal (reiserfs_filsys_t * fs)
 	return 0;
     }
 
-    if (control.mount_id != newest.mount_id) {
-	reiserfs_warning (stderr, "Journal header's mountid: %lu\n"
-	  "latest transaction found is of mountid %lu\n", control.mount_id, newest.mount_id);
-        if (user_confirmed (stderr, "Should those transactions be replayed?(Y/n)[n]", "Y\n")) {
-            control.mount_id = newest.mount_id;
-//            control.trans_id = 0;
-        }
-    }
-	
+    /*  Smth strange with journal header or journal. We cannot say for sure what was the last 
+	replaied transaction, but relying on JH data is preferable. */
+
     replayed = 0;
-    while (1) {
-	if (cur.mount_id == control.mount_id && cur.trans_id > control.trans_id) {
-            reiserfs_warning (stderr, "trans replayed: mountid %lu, transid %lu, desc %lu, "
+    while (cur.mount_id != control.mount_id || cur.trans_id != control.trans_id) {
+	ret = next_transaction (fs, &cur, newest);
+	if (ret != TRANS_FOUND)
+	    break;
+    }
+
+    while (ret == TRANS_FOUND) {
+	ret = next_transaction (fs, &cur, newest);
+	
+	if (ret == TRANS_NOT_FOUND)
+	    break;
+	
+	if (cur.mount_id == control.mount_id && cur.trans_id == control.trans_id + 1) {
+	    if (!transaction_check_content(fs, &cur)) {
+		reiserfs_warning (stderr, "Trans broken: mountid %lu, transid %lu, desc %lu, "
+		    "len %lu, commit %lu, next trans offset %lu\n", cur.mount_id, cur.trans_id, 
+		    cur.desc_blocknr, cur.trans_len, cur.commit_blocknr, cur.next_trans_offset);
+		break;
+	    }
+
+            reiserfs_warning (stderr, "Trans replayed: mountid %lu, transid %lu, desc %lu, "
                 "len %lu, commit %lu, next trans offset %lu\n",
 		cur.mount_id, cur.trans_id, cur.desc_blocknr, cur.trans_len,
 		cur.commit_blocknr, cur.next_trans_offset);
 	    replay_one_transaction (fs, &cur);
 	    update_journal_header (bh, cur);
+	    control = cur;
             replayed ++;
-	}
-	if ((ret == TRANS_LAST) || ((ret = next_transaction (fs, &cur, newest)) == TRANS_NOT_FOUND) )
+	} else 
 	    break;
     }
+
     reiserfs_warning (stderr, "%d transactions replayed\n", replayed);
 	
     mark_buffer_dirty (fs->fs_super_bh);
