@@ -403,7 +403,7 @@ char timebuf[256];
 
 char * timestamp (time_t t)
 {
-    strftime (timebuf, 256, "%m/%d/%Y %T", localtime (&t));
+    strftime (timebuf, 256, "%d/%Y %T", localtime (&t));
     return timebuf;
 }
 
@@ -607,6 +607,7 @@ int print_super_block (FILE * fp, reiserfs_filsys_t * fs, char * file_name,
     struct reiserfs_super_block * sb = (struct reiserfs_super_block *)(bh->b_data);
     dev_t rdev;
     int format = 0;
+    __u16 state;
 
     if (!does_look_like_super_block (sb))
 	return 1;
@@ -643,7 +644,7 @@ int print_super_block (FILE * fp, reiserfs_filsys_t * fs, char * file_name,
 	reiserfs_warning (fp, "Root block: %u\n", get_sb_root_block (sb));
     }
     reiserfs_warning (fp, "Filesystem is %scleanly umounted\n",
-		      (get_sb_umount_state (sb) == REISERFS_CLEANLY_UMOUNTED) ? "" : "NOT ");
+		      (get_sb_umount_state (sb) == FS_CLEANLY_UMOUNTED) ? "" : "NOT ");
 
     if (short_print)
     	return 0;
@@ -656,14 +657,23 @@ int print_super_block (FILE * fp, reiserfs_filsys_t * fs, char * file_name,
     print_journal_params (fp, sb_jp (sb));
     reiserfs_warning (fp, "Blocks reserved by journal: %u\n",
 		      get_sb_reserved_for_journal (sb));
-    reiserfs_warning (fp, "Fs state field: 0x%x\n", get_sb_fs_state (sb));
+    state = get_sb_fs_state (sb);
+    reiserfs_warning (fp, "Fs state field: 0x%x:\n", state);
+    if ((state & FS_FATAL) == FS_FATAL)
+	reiserfs_warning (fp, "\tFATAL corruptions exist.\n");
+    if ((state & FS_ERROR) == FS_ERROR)
+	reiserfs_warning (fp, "\t some corruptions exist.\n");
+    if ((state & IO_ERROR) == IO_ERROR)
+	reiserfs_warning (fp, "\tI/O corruptions exist.\n");
+
     reiserfs_warning (fp, "sb_version: %u\n", get_sb_version (sb));
     if (format == 2) {
         reiserfs_warning (fp, "inode generation number: %u\n", get_sb_v2_inode_generation (sb));
         reiserfs_warning (fp, "UUID: %U\n", sb->s_uuid);
         reiserfs_warning (fp, "LABEL: %.16s\n", sb->s_label);
         reiserfs_warning (fp, "Set flags in SB:\n");
-        reiserfs_warning (fp, "\t%s\n", ((get_sb_v2_flag (sb, reiserfs_attrs_cleared)) ? "ATTRIBUTES CLEAN" : ""));
+	if ((get_sb_v2_flag (sb, reiserfs_attrs_cleared)))
+	    reiserfs_warning (fp, "\tATTRIBUTES CLEAN\n");
     }
 
     return 0;
@@ -812,13 +822,13 @@ static void print_bmap_block (FILE * fp, int i, unsigned long block, char * map,
 
     blocks = blocksize * 8;
 
-    if (test_bit (0, map)) {
+    if (misc_test_bit (0, map)) {
 	/* first block addressed by this bitmap block is used */
 	ones ++;
 	if (!silent)
 	    reiserfs_warning (fp, "Busy (%d-", i * bits);
 	for (j = 1; j < blocks; j ++) {
-	    while (test_bit (j, map)) {
+	    while (misc_test_bit (j, map)) {
 		ones ++;
 		if (j == blocks - 1) {
 		    if (!silent)
@@ -830,7 +840,7 @@ static void print_bmap_block (FILE * fp, int i, unsigned long block, char * map,
 	    if (!silent)
 		reiserfs_warning (fp, "%d) Free(%d-", j - 1 + i * bits, j + i * bits);
 
-	    while (!test_bit (j, map)) {
+	    while (!misc_test_bit (j, map)) {
 		zeros ++;
 		if (j == blocks - 1) {
 		    if (!silent)
@@ -844,6 +854,7 @@ static void print_bmap_block (FILE * fp, int i, unsigned long block, char * map,
 
 	    j --;
 	end:
+	    /* to make gcc 3.2 do not sware here */;
 	}
     } else {
 	/* first block addressed by this bitmap is free */
@@ -852,7 +863,7 @@ static void print_bmap_block (FILE * fp, int i, unsigned long block, char * map,
 	    reiserfs_warning (fp, "Free (%d-", i * bits);
 	for (j = 1; j < blocks; j ++) {
 	    k = 0;
-	    while (!test_bit (j, map)) {
+	    while (!misc_test_bit (j, map)) {
 		k ++;
 		if (j == blocks - 1) {
 		    if (!silent)
@@ -867,7 +878,7 @@ static void print_bmap_block (FILE * fp, int i, unsigned long block, char * map,
 		reiserfs_warning (fp, "%d) Busy(%d-", j - 1 + i * bits, j + i * bits);
 	    
 	    k = 0;
-	    while (test_bit (j, map)) {
+	    while (misc_test_bit (j, map)) {
 		ones ++;
 		if (j == blocks - 1) {
 		    if (!silent)
@@ -883,6 +894,7 @@ static void print_bmap_block (FILE * fp, int i, unsigned long block, char * map,
 	
 	    j --;
 	end2:
+	    /* to make gcc 3.2 do not sware here */;
 	}
     }
 
