@@ -46,15 +46,15 @@ int is_block_count_correct (unsigned long block_of_super_block, int block_size,
 }
 
 /* read super block. fixme: only 4k blocks, pre-journaled format
-   is refused. Journal and bitmap are to be opened separately */
-reiserfs_filsys_t * reiserfs_open (char * filename, int flags, int *error, void * vp)
+   is refused. Journal and bitmap are to be opened separately.
+   skip_check is set to 1 if checks of openned SB should be omitted.*/
+reiserfs_filsys_t * reiserfs_open (char * filename, int flags, int *error, void * vp, int skip_check)
 {
     reiserfs_filsys_t * fs;
     struct buffer_head * bh;
     struct reiserfs_super_block * sb;
     int fd, i;
 
-
     /* convert root dir key and parent root dir key to little endian format */
     make_const_keys ();
 
@@ -80,7 +80,7 @@ reiserfs_filsys_t * reiserfs_open (char * filename, int flags, int *error, void
 	} else {
 	    sb = (struct reiserfs_super_block *)bh->b_data;
 	    
-	    if (does_look_like_super_block (sb))
+	    if (is_any_reiserfs_magic_string(sb))
 		goto found;
 
 	    /* reiserfs signature is not found at the i-th 4k block */
@@ -88,9 +88,9 @@ reiserfs_filsys_t * reiserfs_open (char * filename, int flags, int *error, void
 	}
     }
 
-    reiserfs_warning (stderr, "reiserfs_open: neither new nor old reiserfs format "
-		      "found on %s\n", filename);
-
+    reiserfs_warning(stderr, 
+	"\nreiserfs_open: the reiserfs superblock cannot be found on %s.\n", filename);
+    
     if (error)
 	*error = 0;
     freemem (fs);
@@ -99,6 +99,35 @@ reiserfs_filsys_t * reiserfs_open (char * filename, int flags, int *error, void
     return fs;
 
  found:
+
+    if (!skip_check) {
+	/* A few checks of found super block. */
+	struct buffer_head *tmp_bh;
+	
+	if (!is_blocksize_correct(get_sb_block_size(sb))) {
+	    reiserfs_warning(stderr, "reiserfs_open: a superblock with wrong parameters "
+		"was found in the block (%d).\n", i);
+	    freemem (fs);
+	    close (fd);
+	    brelse(bh);
+	    return NULL;
+	}
+    
+	tmp_bh = bread (fd, get_sb_block_count(sb) - 1, get_sb_block_size(sb));
+	
+	if (!tmp_bh) {
+	    reiserfs_warning (stderr, "\n%s: Your partition is not big enough to contain the \n"
+		    "filesystem of (%lu) blocks as was specified in the found super block.\n", 
+		    __FUNCTION__,  get_sb_block_count(sb) - 1);
+	    
+	    freemem (fs);
+	    close (fd);
+	    brelse(bh);
+	    return NULL;
+	}
+	
+	brelse(tmp_bh);
+    }
    
     fs->fs_blocksize = get_sb_block_size (sb);
     
@@ -167,8 +196,8 @@ reiserfs_filsys_t * reiserfs_create (char * filename,
 
     fs->fs_dev = open (filename, O_RDWR | O_LARGEFILE);
     if (fs->fs_dev == -1) {
-	reiserfs_warning (stderr, "reiserfs_create: could not open %s: %m\n",
-			  filename);
+	reiserfs_warning (stderr, "reiserfs_create: could not open %s: %s\n",
+			  filename, strerror(errno));
 	freemem (fs);
 	return 0;
     }
@@ -262,24 +291,15 @@ int spread_bitmaps (reiserfs_filsys_t * fs)
 /* 0 means: do not guarantee that fs is consistent */
 int reiserfs_is_fs_consistent (reiserfs_filsys_t * fs)
 {
-    if (get_sb_umount_state (fs->fs_ondisk_sb) == REISERFS_CLEANLY_UMOUNTED &&
-	get_sb_fs_state (fs->fs_ondisk_sb) == REISERFS_CONSISTENT)
+    if (get_sb_umount_state (fs->fs_ondisk_sb) == FS_CLEANLY_UMOUNTED &&
+	get_sb_fs_state (fs->fs_ondisk_sb) == FS_CONSISTENT)
 	return 1;
     return 0;
 }
 
-
-void reiserfs_reopen (reiserfs_filsys_t * fs, int flag)
-{
-    reiserfs_only_reopen (fs, flag);
-    reiserfs_reopen_journal (fs, flag);
-
-}
-
-
 /* flush bitmap, brelse super block, flush all dirty buffers, close and open
    again the device, read super block */
-void reiserfs_only_reopen (reiserfs_filsys_t * fs, int flag)
+static void reiserfs_only_reopen (reiserfs_filsys_t * fs, int flag)
 {
     unsigned long super_block;
 
@@ -291,11 +311,11 @@ void reiserfs_only_reopen (reiserfs_filsys_t * fs, int flag)
     
     invalidate_buffers (fs->fs_dev);
     if (close (fs->fs_dev))
-	die ("reiserfs_reopen: closed failed: %m");
+	die ("reiserfs_reopen: closed failed: %s", strerror(errno));
     
     fs->fs_dev = open (fs->fs_file_name, flag | O_LARGEFILE);
     if (fs->fs_dev == -1)
-	die ("reiserfs_reopen: could not reopen device: %m");
+	die ("reiserfs_reopen: could not reopen device: %s", strerror(errno));
 
     fs->fs_super_bh = bread (fs->fs_dev, super_block, fs->fs_blocksize);
     if (!fs->fs_super_bh)
@@ -309,6 +329,11 @@ void reiserfs_only_reopen (reiserfs_filsys_t * fs, int flag)
 	fs->fs_dirt = 0;
 }
 
+void reiserfs_reopen (reiserfs_filsys_t * fs, int flag)
+{
+    reiserfs_only_reopen (fs, flag);
+    reiserfs_reopen_journal (fs, flag);
+}
 
 int is_opened_rw (reiserfs_filsys_t * fs)
 {
@@ -1009,7 +1034,7 @@ char * make_entry (char * entry, char * name, struct key * key, __u32 offset)
 /* add new name into a directory. If it exists in a directory - do
    nothing */
 int reiserfs_add_entry (reiserfs_filsys_t * fs, struct key * dir, char * name, int name_len,
-			struct key * key, int fsck_need)
+			struct key * key, __u16 fsck_need)
 {
     struct item_head entry_ih = {{0,}, };
     char * entry;
