@@ -214,7 +214,11 @@ int main (int argc, char **argv)
     int Is_journal_or_maxtrans_size_specified = 0;
 
     program_name = strrchr( argv[ 0 ], '/' );
-    program_name = program_name ? ++ program_name : argv[ 0 ];
+
+    if (program_name)
+	program_name++;
+    else 
+	program_name = argv[ 0 ];
     
     print_banner (program_name);
 
@@ -339,7 +343,7 @@ int main (int argc, char **argv)
 		     "or choose the option --no-journal-awailable");
 	    return 1;
     }
-    fs = reiserfs_open (device_name, O_RDONLY, 0, NULL);
+    fs = reiserfs_open (device_name, O_RDONLY, 0, NULL, 0);
     if (no_reiserfs_found(fs)) {
 	message ("Cannot open reiserfs on %s", device_name);
         return 1;
@@ -370,7 +374,7 @@ int main (int argc, char **argv)
     /* make sure filesystem is not mounted */
     if (is_mounted (fs->fs_file_name)) {
 	/* fixme: it can not be mounted, btw */
-        message ("can not rebuild journal of mounted filesystem");
+        message ("Reiserfstune is not allowed to be run on mounted filesystem.");
 	reiserfs_close (fs);
         return 1;
     }
